@@ -170,7 +170,7 @@ if TYPE_CHECKING:
             "contexts": dict[str, dict[str, object]],
             "dist": str,
             "duration": Optional[float],
-            "environment": str,
+            "environment": Optional[str],
             "errors": list[dict[str, Any]],  # TODO: We can expand on this type
             "event_id": str,
             "exception": dict[
@@ -188,7 +188,7 @@ if TYPE_CHECKING:
             "monitor_slug": Optional[str],
             "platform": Literal["python"],
             "profile": object,  # Should be sentry_sdk.profiler.Profile, but we can't import that here due to circular imports
-            "release": str,
+            "release": Optional[str],
             "request": dict[str, object],
             "sdk": Mapping[str, object],
             "server_name": str,
@@ -269,7 +269,7 @@ if TYPE_CHECKING:
         "metric_bucket",
         "monitor",
         "span",
-        "log",
+        "log_item",
     ]
     SessionStatus = Literal["ok", "exited", "crashed", "abnormal"]
 
